package repolist

import (
	"go.wit.com/lib/gui/repostatus"
	"go.wit.com/log"
)

func (r *RepoList) Hidden() bool {
	return r.reposbox.Hidden()
}

func (r *RepoList) Show() {
	r.reposbox.Show()
}

func (r *RepoList) Hide() {
	r.reposbox.Hide()
}

func (r *RepoList) FindRepo(path string) *Repo {
	repo, _ := me.allrepos[path]
	return repo
}

func (r *RepoList) AllRepos() []*Repo {
	var all []*Repo
	for _, repo := range me.allrepos {
		all = append(all, repo)
	}
	return all
}

// deprecate this
func AllRepos() []*Repo {
	var all []*Repo
	for _, repo := range me.allrepos {
		all = append(all, repo)
	}
	return all
}

// a human readable state of the current repo
func (r *Repo) State() string {
	return r.gitState.String()
}

func (r *Repo) Scan() bool {
	return r.NewScan()
}

// returns a name for human consuption only
// todo: implement nicknames
func (rs *Repo) Name() string {
	if rs.Status.IsGoLang() {
		return rs.Status.GoPath()
	}
	return rs.Status.Path()
}

func (r *Repo) Exists(s string) bool {
	return false
}

func (r *Repo) GoPath() string {
	return r.Status.GoPath()
}

func (r *Repo) CheckDirty() bool {
	return r.Status.CheckDirty()
}

func (r *Repo) IsDirty() bool {
	return r.Status.IsDirty()
}

func (r *Repo) ReadOnly() bool {
	return r.Status.ReadOnly()
}

func (r *Repo) LastTag() string {
	return r.Status.GetLastTagVersion()
}

// returns the state of the GO go.mod and go.sum files
// this is used to tell if they are valid and correctly reflect
// the versions of the other GUI packages
// at this point in time, there is _NO_ way to be check our
// be sure that anything will run with older versions
// because this are changing too often at this point
// TODO: revisit this in 2025 or 2026
func (r *Repo) GoState() string {
	if r == nil {
		log.Info("GoState() r == nil")
		return "goState == nil"
	}
	if r.goState == nil {
		log.Info("GoState() r.goState == nil")
		return "goState == nil"
	}
	return r.goState.String()
}

func (r *Repo) SetGoState(s string) {
	if r == nil {
		log.Info("SetGoState() r == nil")
		return
	}
	if r.goState == nil {
		log.Info("goState == nil")
		return
	}
	r.goState.SetText(s)
}

func (r *Repo) IsPerfect() bool {
	if r.gitState.String() == "PERFECT" {
		return true
	}
	if r.gitState.String() == "unchanged" {
		return true
	}
	return false
}

func (r *Repo) RunCmd(cmd []string) (error, string) {
	return r.Status.RunCmd(cmd)
}

func (r *Repo) AllTags() []*repostatus.Tag {
	return r.Status.Tags.ListAll()
}

func (r *Repo) TagsBox() *repostatus.GitTagBox {
	return r.Status.Tags
}

// todo, fix bool return for deletetag()
func (r *Repo) DeleteTag(t *repostatus.Tag) bool {
	r.Status.DeleteTag(t)
	return true
}
